"""MainWindow of Beamline console.

Based on P22 EC by sebastian.piec@desy.de and yury.matveev@desy.de
"""

import logging
import os
import sys

import psutil

from PyQt5 import QtWidgets, QtCore

from beamline_console.constants import APP_NAME, LOG_FORMATTER

from beamline_console.snapshot.snapshotwidget import SnapshotWidget
from beamline_console.devicecontrol.beamlinewidget import BeamlineWidget
from beamline_console.beamlinehal import BeamlineHAL
from beamline_console.logger.loggingwidget import LoggingWidget
from beamline_console.logger.guilogger import GuiLogger
from beamline_console.utils.aboutdialog import AboutDialog
from beamline_console.utils.errordialog import ErrormsgLogger
from beamline_console.utils.settings import Settings
from beamline_console.utils.input_dialog import InputDialog

from beamline_console.gui.MainWindow_ui import Ui_MainWindow

logger = logging.getLogger(APP_NAME)
gui_settings = QtCore.QSettings(APP_NAME)


# ----------------------------------------------------------------------
class ExperimentalControl(QtWidgets.QMainWindow):
    """
    """
    windowClosed = QtCore.pyqtSignal()

    # ----------------------------------------------------------------------
    def __init__(self, options):
        """
        """
        super().__init__()

        self._childWindows = []
        self._dockList = []

        self._options = options

        # avoid running multiple instances of the app
        # self._lockFileName = "{}.lock".format(str(options.beamlineID).lower())
        # assertUniqueInstance(self._lockFileName, self)

        # process main settings
        main_folder = os.path.join(os.path.expanduser('~'), '.beamline_console')

        self.settings = Settings(os.path.join(main_folder, "main.cfg"))

        errh = ErrormsgLogger()
        errh.setFormatter(LOG_FORMATTER)
        logger.addHandler(errh)

        self._gui_logger = GuiLogger()
        self._gui_logger.setFormatter(LOG_FORMATTER)
        logger.addHandler(self._gui_logger)

        self.beamline_hal = BeamlineHAL(self, options.user)

        # GUI
        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)
        menu_bar = self.menuBar()

        self.action_user = QtWidgets.QAction('Enable superuser', self)
        self.action_user.triggered.connect(self.switch_user)
        menu_bar.addAction(self.action_user)

        self.menu_view = QtWidgets.QMenu('View')
        menu_bar.addMenu(self.menu_view)

        action_about = QtWidgets.QAction('About', self)
        action_about.triggered.connect(self._show_about)
        menu_bar.addAction(action_about)

        action_quit = QtWidgets.QAction('Quit', self)
        action_quit.triggered.connect(self.exit_program)
        menu_bar.addAction(action_quit)

        self.setCentralWidget(None)

        self.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks |
                            QtWidgets.QMainWindow.AllowNestedDocks |
                            QtWidgets.QMainWindow.AllowTabbedDocks)

        self.widgets = []

        try:
            self.restoreGeometry(gui_settings.value("mainWindow/geometry"))
            self.restoreState(gui_settings.value("mainWindow/state"))
        except:
            pass

        self._init_status_bar()

        self._refresh_status_timer = QtCore.QTimer(self)
        self._refresh_status_timer.timeout.connect(self._refresh_status_bar)
        self._refresh_status_timer.start(1000)

    # ----------------------------------------------------------------------
    def finish_init(self):
        try:
            logging_widget = LoggingWidget(self)
            self._add_dock(logging_widget, "Logs")
            logging_widget.load_ui_settings(gui_settings)
            self._gui_logger.emit_record.connect(logging_widget.get_record)
            self.widgets.append(logging_widget)
        except Exception as err:
            logger.error(f"Cannot initialize log widget: {err}")
            return

        try:
            snapshot_widget = SnapshotWidget(self)
            self._add_dock(snapshot_widget, "Beamline Snapshots")
            snapshot_widget.load_ui_settings(gui_settings)
            self.widgets.append(snapshot_widget)
        except Exception as err:
            logger.error(f"Cannot initialize snapshots widget: {err}")
            return

        try:
            beamline_widget = BeamlineWidget(self)
            self._add_dock(beamline_widget, "Beamline Control")
            beamline_widget.load_ui_settings(gui_settings)
            self.widgets.append(beamline_widget)
        except Exception as err:
            logger.error(f"Cannot initialize control widget: {err}")
            return

        try:
            beamline_widget.finish_init()
        except Exception as err:
            logger.error(f"Cannot initialize control widget: {err}")
            return

        # Cross-connection signals
        beamline_widget.save_snapshot.connect(snapshot_widget.save_snapshot)
        beamline_widget.device_selected.connect(snapshot_widget.device_selected)

        snapshot_widget.motors_moved.connect(beamline_widget.sync_control_widget)

        dev_list = list(self.beamline_hal.device_map.keys())
        if len(dev_list) == 1:
            snapshot_widget.device_selected(dev_list)

        self.widgets = [beamline_widget, snapshot_widget, logging_widget]

        logger.info("Beamline console successfully initialized")

    # ----------------------------------------------------------------------
    def _add_dock(self, widget, label):
        """
        """

        dock = QtWidgets.QDockWidget(label)
        dock.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        dock.setFocusPolicy(QtCore.Qt.StrongFocus)
        dock._currentWindow = '0'
        dock.setObjectName("{}Dock".format("".join(label.split())))
        dock.setWidget(widget)

        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dock)
        self._dockList.append(dock)
        self.menu_view.addAction(dock.toggleViewAction())

    # ----------------------------------------------------------------------
    def _stop_beamline(self):
        """
        """
        self.beamline_hal.stop_all()
        self.beamline_hal.safe_close()
     
    # ----------------------------------------------------------------------  
    def _save_ui_settings(self):
        """Save GUI state using QT settings mechanism.
        """

        for widget in self.widgets:
            widget.save_ui_settings(gui_settings)

        gui_settings.setValue("mainWindow/state", self.saveState())
        gui_settings.setValue("mainWindow/geometry", self.saveGeometry())

    # ----------------------------------------------------------------------
    def _init_status_bar(self):

        # move progressbar TODO
        self.pb_movement = QtWidgets.QProgressBar()
        self.pb_movement.setMaximumSize(2000, 25)

        # display info on the status bar
        lb_process_id = QtWidgets.QLabel(f"pid {os.getpid()} |")
        lb_process_id.setStyleSheet("QLabel {color: #000066;}")
    
        # resources usage info
        process = psutil.Process(os.getpid())
        mem = float(process.memory_info().rss) / (1024. * 1024.)
        cpu = process.cpu_percent()

        self._lb_resources_status = QtWidgets.QLabel(f"| {mem:.2f}MB | CPU {cpu} %")

        self.statusBar().addPermanentWidget(QtWidgets.QLabel("Movement status: "))
        self.statusBar().addPermanentWidget(self.pb_movement)
        self.statusBar().addPermanentWidget(lb_process_id)
        self.statusBar().addPermanentWidget(self._lb_resources_status)

    # ----------------------------------------------------------------------
    def switch_user(self):
        """
        """
        try:
            if self.beamline_hal.access_level != 'superuser':
                if InputDialog(mode="pin", super_user_password=int(self.settings.option("general", "superuser_pin")),
                               max_attempts=int(self.settings.option("general", "max_attempts"))).exec():
                    self.beamline_hal.access_level_changed('superuser')
                    for widget in self.widgets:
                        widget.access_level_changed('superuser')
                    self.action_user.setText('Disable superuser')
            else:
                self.beamline_hal.access_level_changed('user')
                for widget in self.widgets:
                    widget.access_level_changed('user')
                self.action_user.setText('Enable superuser')

        except Exception as err:
            logger.error(f"Authorization Failed: {repr(err)}", exc_info=sys.exc_info())

    # ----------------------------------------------------------------------
    def _refresh_status_bar(self):
        """
        """
        process = psutil.Process(os.getpid())
        mem = float(process.memory_info().rss) / (1024. * 1024.)
        cpu = psutil.cpu_percent()      # of the whole system?

        self._lb_resources_status.setText(f" {mem:.2f}MB | CPU {cpu} %")

        self.pb_movement.setValue(self.beamline_hal.progress())

    # ----------------------------------------------------------------------
    def _show_about(self):
        """
        """
        AboutDialog(self).show()
    
    # ---------------------------------------------------------------------- 
    def closeEvent(self, event):
        """Process "close event" (e.g. emitted when window's cross is clicked).
        """
        if self._clean_exit():
            event.accept()  
        else:  
            event.ignore()  

    # ----------------------------------------------------------------------
    def exit_program(self, force=False):
        """Called when Quit action is triggered.
        """
        if self._clean_exit(force):
            QtWidgets.qApp.quit()

    # ----------------------------------------------------------------------
    def _clean_exit(self, force=False):
        """
        """ 
        really_quit = True if force else self._really_quit()
        if really_quit:
            self._refresh_status_timer.stop()
            self._stop_beamline()
            # unlockAppInstance(self._lockFileName)
            self._save_ui_settings()
            QtWidgets.qApp.clipboard().clear()
            return True

        return False

    # ----------------------------------------------------------------------  
    @staticmethod
    def _really_quit():
        """Make sure that the user wants to quit this program.
        """
        msg_box = QtWidgets.QMessageBox()
        msg_box.setWindowTitle("Quit")
        msg_box.setText(f"Do you want to exit beamline console?")
        msg_box.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)

        return msg_box.exec_() == QtWidgets.QMessageBox.Yes



"""Beamline Hardware Abstraction Layer.

Extra software layer on top of the hardware middleware (e.g. TANGO)

Provides a few additional services:
    - access control (e.g. restrict users to move only selected motors)
    - device history (e.g. previous positions of a selected motor/device)
    - additional user limits, on top of Tango
    - collision detection
    - ...
"""

import collections
import os
import sys
import numpy as np

from datetime import datetime
from xml.dom.minidom import parseString

import tango

from beamline_console.logger.logit import Logit
from beamline_console.devices.motordevice import MotorBasedDevice
from beamline_console.snapshot.beamsnapshot import BeamlineSnapshot


# ----------------------------------------------------------------------
class BeamlineHAL(Logit):
    """
    """
    SOFTWARE_VERSION = "1.0"

    TIME_STAMP_FMT = "%Y-%m-%d %H:%M:%S"
  
    HISTORY_BUFFER_SIZE = 1000

    # ---------------------------------------------------------------------- 
    def __init__(self, parent, initial_user):
        """
        """
        Logit.__init__(self)

        self.settings = parent.settings
        self.device_map = collections.OrderedDict()     # (dev_name, dev_handle)

        self.file_name = os.path.join(os.path.join(os.path.expanduser('~'), '.beamline_console'), "devices.xml")

        self.access_level = initial_user

        self.load_devices(self.file_name)

        # use ring buffer TODO
        # memorize positions of devices and motors
        self.history = []       # (dev_name, motor_name, current_pos, target_pos)

        self._timeout = 0
    
    # ----------------------------------------------------------------------
    def __del__(self):
        """ 
        """
        self.group_action("stop_motors")
        self.logger.debug("BeamlineHAL object released successfully!")

    # ----------------------------------------------------------------------
    def access_level_changed(self, access_level):
        self.logger.info(f"Access level changed to {access_level}")
        self.access_level = access_level
        for device in self.device_map:
            if hasattr(device, "access_level_changed"):
                device.access_level_changed(access_level)

    # ----------------------------------------------------------------------
    def progress(self):
        progress = []
        for device in self.device_map.values():
            device_progress = device.progress()
            if device_progress:
                progress.append(device_progress)

        if progress:
            return min(progress)*100
        else:
            return 0

    # ----------------------------------------------------------------------
    def device(self, name):
        """
        Args:
            name (str)
        Returns:
            handle to a given device.
        """
        return self.device_map[str(name)]
    
    # ---------------------------------------------------------------------- 
    def group_action(self, func_name):
        """Calls function for all controlled devices
        """
        self.logger.info(f"Group action {func_name} requested")

        for device_handle in list(self.device_map.values()):
            try:
                getattr(device_handle, func_name)()
      
            except AttributeError as err:
                if func_name != 'safe_close':
                    self.logger.error(f"No {func_name} function available {str(err)}", exc_info=sys.exc_info())
                else:
                    self.logger.debug(f"No {func_name} function available {str(err)}")
 
    # ----------------------------------------------------------------------
    def _get_motor_handle(self, motor_name):
        device_name, motor_name = str(motor_name).split("/")

        device_handle = self.device_map[device_name]
        if isinstance(device_handle, MotorBasedDevice):
            motor_handle = device_handle.motor(motor_name)
            if not motor_handle:
                raise RuntimeError(f"Invalid motor name ({device_name}/{motor_name})")
        else:
            raise RuntimeError(f'MotorBasedDevice object expected (given {device_name})')

        return motor_handle

    # ----------------------------------------------------------------------
    def move_motor(self, motor_name, value, scale="absolute", mode="async"):
        """
        Args:
            motor_name (str): name of motor including device (e.g. "manipulator/x")
            value (float): "target position" or "delta" (depending on "scale")
            scale (str): "relative" or "absolute"
            mode (str): "sync" or "async" or 'snapshot'

            Examples:
                hal.move_motor("manipulator/x", 20.2, "absolute")
                hal.move_motor("manipulator/y", 0.2, "relative", "sync")
        """

        motor_handle = self._get_motor_handle(motor_name)
        current_pos = motor_handle.position()

        if scale.lower() == "relative":
            target_pos = current_pos + value
        elif scale.lower() == "absolute":
            target_pos = value
        else:
            raise RuntimeError(f"Invalid scale {scale}")

        if not motor_handle.access_guaranteed():
            raise RuntimeError(f"Current user is not allowed to move {motor_name}")

        # memorize move
        timestamp = datetime.now().strftime(self.TIME_STAMP_FMT)
        self.history.append((motor_name, current_pos, target_pos, timestamp))
        self.logger.info(f"Moving {motor_name} from "
                         f"{current_pos:.{motor_handle.decimals}f} to {target_pos:.{motor_handle.decimals}f}")

        motor_handle.move_absolute(target_pos, mode)

    # ----------------------------------------------------------------------
    def stop_motor(self, motor_name):
        self._get_motor_handle(motor_name).stop()

    # ----------------------------------------------------------------------
    def move_motor_eta(self, motor_name, value, scale):
        """
        Returns:
            (int) estimated time of mv command completion in seconds
        """
        raise NotImplementedError("Not implemented")

    # ----------------------------------------------------------------------
    def move_snapshot(self, filename, mode="sync"):
        """Move beamline's motors to a given state.

        NOTE that movement is performed in a separate thread (special
        treatment of exceptions comming from the thread is needed).
        """
        snapshot = BeamlineSnapshot(self, filename)

        snapshot.validate()   # in case of error RuntimeError is thrown
        snapshot.start()
    
        if mode != "async":      # ??? TODO otherwise don't wait?
            snapshot.join()
  
            exc_type, exc_value = snapshot.last_exception()
            if exc_type:
                raise RuntimeError(f"Beamline snapshot exception, {exc_type}, {exc_value}")

    # ----------------------------------------------------------------------
    def move_group(self, move_group):
        """Move group of motors in async mode.

        e.g. tuple
            (device_name, motor_name, target_position) -> take into account sync/async! TODO
        """
        for move_tuple in move_group:
            dev_name, motor_name, position = move_tuple[:3]
            full_name = f"{dev_name}/{motor_name}"

            try: 
                self.move_motor(full_name, position, "absolute", "async")

            except (RuntimeError, tango.DevFailed) as err:
                self.stop_all()
                raise               # propagate the exception somehow nicer...
  
    # ----------------------------------------------------------------------
    def stop_all(self):
        """Stop all moving motors.
        """
        self.group_action("stop_motors")

    # ----------------------------------------------------------------------
    def safe_close(self):
        """Stop all moving motors.
        """
        self.group_action("safe_close")

    # ----------------------------------------------------------------------
    def load_devices(self, file_name):
        """
        """
        self.file_name = file_name
        self.logger.info(f"Loading device file {file_name}")
  
        with open(file_name) as dev_file:
            dom = parseString(dev_file.read())
        root_elements = dom.getElementsByTagName("beamline_settings")
        if len(root_elements) > 0:
            root_element = root_elements[0]
            self._validate_compatibility(root_element)
    
            # clean-up first (important when Beamline object is reloaded)
            self.device_map = collections.OrderedDict()

            device_list = root_element.getElementsByTagName("device")

            for dev_node in device_list:
                dev_name = dev_node.getAttribute("name")            # alias

                if dev_name in list(self.device_map.keys()):
                    raise RuntimeError(f'Non-unique device name found: {dev_name}')

                try:
                    device = MotorBasedDevice(dev_name, self)
                    device.from_xml(dev_node)
                    status = device.initialize()
                    if status:
                        self.device_map[dev_name] = device
                    else:
                        self.logger.error(f'Cannot make {dev_name}')

                except Exception as ex:
                    self.logger.error(f'Cannot make {dev_name}', exc_info=sys.exc_info())
                    raise ex
        else:
            raise RuntimeError('No root node "beamline_settings" found!')

    # ----------------------------------------------------------------------
    def _validate_compatibility(self, xml_node):
        """
        """
        software_version = xml_node.getAttribute("version")

        if software_version != self.SOFTWARE_VERSION:
            self.logger.warning(f"Probable software incompatibility! ({software_version} vs. {self.SOFTWARE_VERSION})")

    # ----------------------------------------------------------------------
    def motor_state(self, motor_name):
        """
        Args:
            (str) absolute motor name (e.g. manipulator/x)
        Returns:
            (str) representing state of a given motor (e.g. "on", "moving")
        """
        device_name, motor_name = motor_name.split("/")

        device_handle = self.device_map[device_name]
        if not isinstance(device_handle, MotorBasedDevice):
            raise RuntimeError(f"MotorBasedDevice object expected ({device_name})")

        motor_handle = device_handle.motor(motor_name)
        if not motor_handle:
            raise RuntimeError(f'Invalid motor name "{motor_name}"')

        return motor_handle.state()

    # ---------------------------------------------------------------------- 
    def motor_position(self, motor_name):
        """
        Example usage:
            beamlineHal.motor_position("manipulator/x")
    
        Returns:
            (float) position of a given motor.
        """
        try:
            device_name, motor_name = motor_name.split("/")

        except ValueError:
            raise RuntimeError(f"Invalid motor name '{motor_name}'!")

        device_handle = self.device_map[device_name]
        if isinstance(device_handle, MotorBasedDevice):

                # get handle to the motor identified by the name
            motor_handle = device_handle.motor(motor_name)
            if not motor_handle:
                raise RuntimeError('Invalid motor name "{}/{}"'.format(device_name,
                                                                       motor_name))

            return np.round(motor_handle.position(), decimals = motor_handle.decimals)

        else:
            raise RuntimeError(f'MotorBasedDevice object expected (device name: "{device_name}")')

    # ----------------------------------------------------------------------
    def list_motors(self):
        """
        Returns:
            [str] complete list of motor names
        """
        motor_list = []

        for dev_name, device in list(self.device_map.items()):
            if isinstance(device, MotorBasedDevice):
                motor_list += [f"{dev_name}/{name}"
                               for name in device.list_motors()]
    
        return motor_list

    # ----------------------------------------------------------------------
    def is_any_moving(self):
        """
        Returns:
            (bool) True if any of constituent motors is currently moving
        """
        return any([handle.state() == "moving" for handle in list(self.device_map.values()) if
                    isinstance(handle, MotorBasedDevice)])

    # ----------------------------------------------------------------------
    def is_move_valid(self, motor_name, position):
        """Check if given motor can be moved to requested target position.
        Args:
            motor_name (str)
            position (float)
        """
        dev_name, motor_name = motor_name.split("/")

        dev_handle = self.device_map[dev_name]
        if not isinstance(dev_handle, MotorBasedDevice):
            return False, f"MotorBasedDevice object expected ({dev_name})"

        motor_handle = dev_handle.motor(motor_name)
        if not motor_handle:
            return False, f"invalid motor name {motor_name}"

        # check access and limits
        return motor_handle.is_move_valid(position)



# yury.matveev@desy.de

from PyQt5 import QtWidgets

from beamline_console.logger.logit import Logit

# ----------------------------------------------------------------------
class BaseWidget(QtWidgets.QWidget, Logit):

    WIDGET_NAME = ""

    # ----------------------------------------------------------------------
    def __init__(self, parent):
        QtWidgets.QWidget.__init__(self, parent)
        Logit.__init__(self)

        self._settings = parent.settings
        self._beamline_hal = parent.beamline_hal

    # ----------------------------------------------------------------------
    def save_ui_settings(self, settings):
        """
        """
        settings.setValue(f"{self.WIDGET_NAME}/geometry", self.saveGeometry())

    # ----------------------------------------------------------------------
    def load_ui_settings(self, settings):
        """
        """
        try:
            self.restoreGeometry(settings.value(f"{self.WIDGET_NAME}//geometry"))
        except:
            pass




"""Graphical representation of device in QGraphicsScene
"""

from PyQt5 import QtCore

from beamline_console.logger.logit import Logit
from beamline_console.devicecontrol.deviceitem.frameitem import FrameItem
from beamline_console.devicecontrol.deviceitem.statusitem import StatusItem
from beamline_console.devicecontrol.deviceitem.titleitem import TitleItem


# ----------------------------------------------------------------------
class DeviceGraphicsItem(QtCore.QObject, Logit):

    # ----------------------------------------------------------------------
    def __init__(self, scene, device_name, geometry):
        QtCore.QObject.__init__(self)
        Logit.__init__(self)
      
        self.device_name = device_name

        self.offsetX = 0
        self.offsetY = 0

        # constituents of "device graphics item"
        self._frame = FrameItem(geometry)
        scene.addItem(self._frame)

        self.status_diode = StatusItem(geometry)
        scene.addItem(self.status_diode)

        self._title = TitleItem(device_name, geometry)
        scene.addItem(self._title)

    # ----------------------------------------------------------------------
    def set_active(self, flag):
        """
        """
        self._frame.setActive(flag)
        self._title.setActive(flag)
        self.status_diode.setActive(flag)
         
    # ----------------------------------------------------------------------
    def set_visible(self, flag):
        """
        """
        self._frame.setVisible(flag)
        self._title.setVisible(flag)
        self.status_diode.setVisible(flag)

    # ----------------------------------------------------------------------
    def updateOffsetX(self, offsetX):
        """
        """
        self.offsetX = offsetX

        self._frame.updateOffsetX(offsetX)
        self._title.updateOffsetX(offsetX)
        self.status_diode.updateOffsetX(offsetX)

    # ----------------------------------------------------------------------
    def intersect(self, point):
        """
        Returns:
            bool, True if given point lies inside selection rectangle
                  (takes into account potential x/y offsets)
        """
        return self._frame.rect().contains(point)



"""Login dialog box. Used to switch users on-the-fly too.
"""

from PyQt5 import QtWidgets

from beamline_console.gui.InputDialog_ui import Ui_InputDialog
from beamline_console.logger.logit import Logit


# ----------------------------------------------------------------------
class InputDialog(QtWidgets.QDialog, Logit):
    """
    """

    # ----------------------------------------------------------------------
    def __init__(self, mode, super_user_password="", max_attempts=0, motor_handle=None):
        QtWidgets.QDialog.__init__(self)
        Logit.__init__(self)

        self._ui = Ui_InputDialog()
        self._ui.setupUi(self)

        self._mode = mode

        self._super_user_password = [int(d) for d in str(super_user_password)]
        self._max_attempts = max_attempts

        self._ui.lb_titel.setText("Enter superuser pin" if mode=="pin" else "New position")
        self._ui.but_punkt.setVisible(mode=="position")
        self._ui.but_sign.setVisible(mode=="position")

        self._motor_handle = motor_handle

        if mode=="position":
            self._ui.but_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel |
                                                QtWidgets.QDialogButtonBox.Ok |
                                                QtWidgets.QDialogButtonBox.Reset)
            self.new_position = motor_handle.position()
        else:
            self._ui.but_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel)

        self._ui.but_box.clicked.connect(self._button_box_clicked)

        self._ui.lb_number.setText("")

        self._attempts = 0
        self._last_typed = []
        self._sign = 1

        for ind in range(10):
            getattr(self._ui, f'but_{ind}').clicked.connect(lambda state, x=ind: self._button_pressed(x))

        self._ui.but_punkt.clicked.connect(lambda: self._new_position(dot=True))
        self._ui.but_sign.clicked.connect(lambda: self._new_position(sign=True))

    # ----------------------------------------------------------------------
    def _button_box_clicked(self, button):
        role = self._ui.but_box.buttonRole(button)
        if role == QtWidgets.QDialogButtonBox.AcceptRole:
            self.apply_new_position()
        elif role == QtWidgets.QDialogButtonBox.RejectRole:
            self.reject()
        elif role == QtWidgets.QDialogButtonBox.ResetRole:
            self._new_position(clear=True)

    # ----------------------------------------------------------------------
    def _button_pressed(self, ind):
        self._last_typed.append(ind)
        if self._mode == "pin":
            self._check_pin()
        else:
            self._new_position()

    # ----------------------------------------------------------------------
    def _check_pin(self):
        if len(self._last_typed) >= len(self._super_user_password):
            if self._last_typed == self._super_user_password:
                self.accept()
            else:
                self._attempts += 1
                self._last_typed = []

                if self._attempts > self._max_attempts:
                    self.reject()

        self._ui.lb_number.setText("".join(["*" for _ in self._last_typed]))

    # ----------------------------------------------------------------------
    def _new_position(self, dot=False, clear=False, sign=False):
        if clear:
            self._last_typed = []
        elif sign:
            self._sign *= -1
        elif dot:
            if '.' not in self._last_typed:
                self._last_typed.append('.')

        value = "-" if self._sign < 0 else ""
        value += "".join([str(v) for v in self._last_typed])
        self._ui.lb_number.setText(value)

    # ----------------------------------------------------------------------
    def apply_new_position(self):
        position = "".join([str(v) for v in self._last_typed])
        try:
            self.new_position = float(position) * self._sign
            self.accept()
        except ValueError:
            self.logger.error(f"Not a number typed: {position}")
            self.reject()



"""Wrapper around CFG files parser
"""

import configparser


# ----------------------------------------------------------------------
class Settings:

    # ----------------------------------------------------------------------
    def __init__(self, filename):
        """
        Args:
            str
        """
        self._config_parser = configparser.ConfigParser()
        
        self.filename = str(filename)
        if self.filename:
            self.load_file(filename)

    # ----------------------------------------------------------------------
    def load_file(self, filename):
        """
        Args:
            str, config file name
        """
        self._config_parser.read(filename)

    # ----------------------------------------------------------------------
    def option(self, section, option):
        """
        Args:
            str, section id
            str, option id
        """
        return self._config_parser.get(section, option)

    # ----------------------------------------------------------------------
    def getDict(self, section):
        """
        Args:
            str, section id
        """
        return self._config_parser._sections[section]

    # ----------------------------------------------------------------------
    def saveOption(self, section, option, value):

        self._config_parser[section][option] = value  # update

        with open(self.filename, 'w') as configfile:  # save
            self._config_parser.write(configfile)

    # ----------------------------------------------------------------------
    def saveOptionSet(self, section, options_values):

        for option, value in list(options_values.items()):
            self._config_parser.set(section, option, value)

        with open(self.filename, 'w') as configfile:  # save
            self._config_parser.write(configfile)





# yury.matveev@desy.de

import logging
import re
import os
from collections import namedtuple
from dateutil.parser import parse

from PyQt5 import QtWidgets, QtCore

from beamline_console import LOG_FOLDER

from beamline_console.base_widget import BaseWidget
from beamline_console.logger.highlightlogs import HighlightLogs
from beamline_console.gui.LoggingWidget_ui import Ui_LoggingWidget

BUFFER_SIZE = 1000

LVL_MAP = {0: logging.DEBUG,
           1: logging.INFO,
           2: logging.WARNING,
           3: logging.ERROR}

NAME_MAP = {"debug": 0,
            "info": 1,
            "warning": 2,
            "error": 3}

# ----------------------------------------------------------------------
class LoggingWidget(BaseWidget):

    WIDGET_NAME = "LoggingWidget"

    # ----------------------------------------------------------------------
    def __init__(self, parent):
        BaseWidget.__init__(self, parent)

        self._logs_buffer = []           # (log_level, message) pairs

        self._ui = Ui_LoggingWidget()
        self._ui.setupUi(self)

        default_log_level = str(parent.settings.option("general", "log_level")).lower()

        if default_log_level in NAME_MAP:
            self._ui.cb_log_level.setCurrentIndex(NAME_MAP[default_log_level])

        self._ui.cb_log_level.currentIndexChanged.connect(self._display_logs)

        self._highlighter = self._setup_text_edit()

        self.read_log()

    # ----------------------------------------------------------------------
    def access_level_changed(self):
        pass

    # ----------------------------------------------------------------------
    def get_record(self, log_level, message):

        if len(self._logs_buffer) > BUFFER_SIZE:        # circular buffer
            self._logs_buffer.pop(0)

        self._logs_buffer.append((log_level, message))
        self._display_logs()

    # ----------------------------------------------------------------------
    def _display_logs(self):
        """
        """
        self._ui.te_logs.clear()

        for level, msg in self._logs_buffer:  #reversed(self._logsBuffer)
            if level >= self._current_level():
                self._ui.te_logs.append(msg)

    # ----------------------------------------------------------------------
    def _current_level(self):
        """
        Returns:
            current logging level (normalized)
        """
        return LVL_MAP[self._ui.cb_log_level.currentIndex()]

    # ----------------------------------------------------------------------
    def _setup_text_edit(self):
        """
        """
        return HighlightLogs(self._ui.te_logs)

    # ----------------------------------------------------------------------
    def read_log(self):

        log_file = os.path.join(LOG_FOLDER, "main.log")

        if os.path.exists(log_file):
            with open(log_file, 'r') as f:
                file_lines = f.readlines()

            if len(file_lines) > BUFFER_SIZE:
                file_lines = file_lines[-BUFFER_SIZE:]

            prog_dialog = QtWidgets.QProgressDialog("", "Cancel", 0, len(file_lines), self)
            prog_dialog.setWindowTitle("Loading logs")
            prog_dialog.setLabelText("Loading logs")
            prog_dialog.setWindowModality(QtCore.Qt.WindowModal)
            prog_dialog.show()

            count = 0
            for line in file_lines:
                data = re.split(" +", line)
                if self.is_date(data[0]):
                    message = f"{' '.join(data[0:2])} {data[3]} {' '.join(data[5:])[:-1]}"
                    self._logs_buffer.append((logging._nameToLevel[data[3]], message))

                QtCore.QCoreApplication.processEvents()
                if prog_dialog.wasCanceled():
                    break
                prog_dialog.setValue(count)
                count += 1

            prog_dialog.close()

            self._display_logs()

    # ----------------------------------------------------------------------
    def is_date(self, string, fuzzy=False):
        """
        Return whether the string can be interpreted as a date.

        :param string: str, string to check for date
        :param fuzzy: bool, ignore unknown tokens in string if True
        """
        try:
            parse(string, fuzzy=fuzzy)
            return True

        except ValueError:
            return False
        
    # ----------------------------------------------------------------------
    def save_ui_settings(self, settings):
        """
        """
        BaseWidget.save_ui_settings(self, settings)

    # ----------------------------------------------------------------------
    def load_ui_settings(self, settings):
        """
        """
        BaseWidget.load_ui_settings(self, settings)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/MotorPropsDialog.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MotorPropsDialog(object):
    def setupUi(self, MotorPropsDialog):
        MotorPropsDialog.setObjectName("MotorPropsDialog")
        MotorPropsDialog.resize(605, 634)
        MotorPropsDialog.setMaximumSize(QtCore.QSize(20000, 20000))
        self.verticalLayout = QtWidgets.QVBoxLayout(MotorPropsDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_7 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_7.setObjectName("label_7")
        self.gridLayout_3.addWidget(self.label_7, 0, 0, 1, 1)
        self.lbMotorName = QtWidgets.QLabel(MotorPropsDialog)
        self.lbMotorName.setMinimumSize(QtCore.QSize(200, 0))
        self.lbMotorName.setStyleSheet("color: rgb(0, 65, 136);")
        self.lbMotorName.setObjectName("lbMotorName")
        self.gridLayout_3.addWidget(self.lbMotorName, 0, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 1, 0, 1, 1)
        self.lbTangoServer = QtWidgets.QLabel(MotorPropsDialog)
        self.lbTangoServer.setMinimumSize(QtCore.QSize(200, 0))
        self.lbTangoServer.setStyleSheet("color: rgb(0, 65, 136);")
        self.lbTangoServer.setObjectName("lbTangoServer")
        self.gridLayout_3.addWidget(self.lbTangoServer, 1, 1, 1, 1)
        self.horizontalLayout_4.addLayout(self.gridLayout_3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.line_2 = QtWidgets.QFrame(MotorPropsDialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.leBacklash = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leBacklash.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leBacklash.setMaxLength(20)
        self.leBacklash.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leBacklash.setObjectName("leBacklash")
        self.gridLayout.addWidget(self.leBacklash, 5, 1, 1, 1)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_3 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_9.addWidget(self.label_3)
        self.label_6 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_6.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_9.addWidget(self.label_6)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem1)
        self.gridLayout.addLayout(self.horizontalLayout_9, 4, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(MotorPropsDialog)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.label_15 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_15.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_3.addWidget(self.label_15)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.gridLayout.addLayout(self.horizontalLayout_3, 1, 0, 1, 1)
        self.leSlewRate = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leSlewRate.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leSlewRate.setMaxLength(20)
        self.leSlewRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leSlewRate.setObjectName("leSlewRate")
        self.gridLayout.addWidget(self.leSlewRate, 3, 1, 1, 1)
        self.leAcceleration = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leAcceleration.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leAcceleration.setMaxLength(20)
        self.leAcceleration.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leAcceleration.setObjectName("leAcceleration")
        self.gridLayout.addWidget(self.leAcceleration, 4, 1, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_2 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_8.addWidget(self.label_2)
        self.label_14 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_14.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_8.addWidget(self.label_14)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem3)
        self.gridLayout.addLayout(self.horizontalLayout_8, 3, 0, 1, 1)
        self.btnCalibrate = QtWidgets.QPushButton(MotorPropsDialog)
        self.btnCalibrate.setMaximumSize(QtCore.QSize(16777215, 28))
        self.btnCalibrate.setIconSize(QtCore.QSize(24, 18))
        self.btnCalibrate.setObjectName("btnCalibrate")
        self.gridLayout.addWidget(self.btnCalibrate, 0, 2, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_4 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_10.addWidget(self.label_4)
        self.label_13 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_13.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_10.addWidget(self.label_13)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.gridLayout.addLayout(self.horizontalLayout_10, 5, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_9 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_7.addWidget(self.label_9)
        self.label_16 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_16.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_7.addWidget(self.label_16)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem5)
        self.gridLayout.addLayout(self.horizontalLayout_7, 0, 0, 1, 1)
        self.leConversion = QtWidgets.QLineEdit(MotorPropsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.leConversion.sizePolicy().hasHeightForWidth())
        self.leConversion.setSizePolicy(sizePolicy)
        self.leConversion.setMinimumSize(QtCore.QSize(150, 0))
        self.leConversion.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leConversion.setMaxLength(20)
        self.leConversion.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leConversion.setObjectName("leConversion")
        self.gridLayout.addWidget(self.leConversion, 1, 1, 1, 1)
        self.leCalibration = QtWidgets.QLineEdit(MotorPropsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.leCalibration.sizePolicy().hasHeightForWidth())
        self.leCalibration.setSizePolicy(sizePolicy)
        self.leCalibration.setMinimumSize(QtCore.QSize(220, 0))
        self.leCalibration.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leCalibration.setMaxLength(20)
        self.leCalibration.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leCalibration.setObjectName("leCalibration")
        self.gridLayout.addWidget(self.leCalibration, 0, 1, 1, 1)
        self.leBaseRate = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leBaseRate.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.leBaseRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leBaseRate.setObjectName("leBaseRate")
        self.gridLayout.addWidget(self.leBaseRate, 6, 1, 1, 1)
        self.label_21 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_21.setObjectName("label_21")
        self.gridLayout.addWidget(self.label_21, 6, 0, 1, 1)
        self.horizontalLayout_11.addLayout(self.gridLayout)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.chbUserLimits = QtWidgets.QCheckBox(MotorPropsDialog)
        self.chbUserLimits.setObjectName("chbUserLimits")
        self.verticalLayout.addWidget(self.chbUserLimits)
        self.frameUserLimits = QtWidgets.QFrame(MotorPropsDialog)
        self.frameUserLimits.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frameUserLimits.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frameUserLimits.setLineWidth(0)
        self.frameUserLimits.setObjectName("frameUserLimits")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.frameUserLimits)
        self.gridLayout_4.setContentsMargins(-1, 0, -1, 0)
        self.gridLayout_4.setHorizontalSpacing(6)
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem6, 0, 4, 1, 1)
        self.leHighLimitUser = QtWidgets.QLineEdit(self.frameUserLimits)
        self.leHighLimitUser.setMaximumSize(QtCore.QSize(140, 16777215))
        self.leHighLimitUser.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leHighLimitUser.setObjectName("leHighLimitUser")
        self.gridLayout_4.addWidget(self.leHighLimitUser, 1, 3, 1, 1)
        self.leLowLimitUser = QtWidgets.QLineEdit(self.frameUserLimits)
        self.leLowLimitUser.setMaximumSize(QtCore.QSize(140, 16777215))
        self.leLowLimitUser.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leLowLimitUser.setObjectName("leLowLimitUser")
        self.gridLayout_4.addWidget(self.leLowLimitUser, 0, 3, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.frameUserLimits)
        self.label_20.setObjectName("label_20")
        self.gridLayout_4.addWidget(self.label_20, 1, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.frameUserLimits)
        self.label_17.setObjectName("label_17")
        self.gridLayout_4.addWidget(self.label_17, 0, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.frameUserLimits)
        self.label_18.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_18.setObjectName("label_18")
        self.gridLayout_4.addWidget(self.label_18, 0, 2, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.frameUserLimits)
        self.label_19.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_19.setObjectName("label_19")
        self.gridLayout_4.addWidget(self.label_19, 1, 2, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem7, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.frameUserLimits)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_11 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 1, 0, 1, 1)
        self.label_12 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_12.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 1, 1, 1, 1)
        self.leHighLimit = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leHighLimit.setMaximumSize(QtCore.QSize(140, 16777215))
        self.leHighLimit.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leHighLimit.setObjectName("leHighLimit")
        self.gridLayout_2.addWidget(self.leHighLimit, 1, 2, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem8, 0, 3, 1, 1)
        self.label_10 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_10.setStyleSheet("color: rgb(124, 120, 120);")
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 0, 0, 1, 1)
        self.leLowLimit = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leLowLimit.setMaximumSize(QtCore.QSize(140, 16777215))
        self.leLowLimit.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leLowLimit.setObjectName("leLowLimit")
        self.gridLayout_2.addWidget(self.leLowLimit, 0, 2, 1, 1)
        self.label_22 = QtWidgets.QLabel(MotorPropsDialog)
        self.label_22.setObjectName("label_22")
        self.gridLayout_2.addWidget(self.label_22, 2, 0, 1, 1)
        self.leDecimals = QtWidgets.QLineEdit(MotorPropsDialog)
        self.leDecimals.setMaximumSize(QtCore.QSize(140, 16777215))
        self.leDecimals.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.leDecimals.setReadOnly(False)
        self.leDecimals.setObjectName("leDecimals")
        self.gridLayout_2.addWidget(self.leDecimals, 2, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_2)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        self.btnAllParams = QtWidgets.QToolButton(MotorPropsDialog)
        self.btnAllParams.setObjectName("btnAllParams")
        self.verticalLayout.addWidget(self.btnAllParams)
        self.line_3 = QtWidgets.QFrame(MotorPropsDialog)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnUndo = QtWidgets.QPushButton(MotorPropsDialog)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/revert.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnUndo.setIcon(icon)
        self.btnUndo.setIconSize(QtCore.QSize(12, 12))
        self.btnUndo.setObjectName("btnUndo")
        self.horizontalLayout.addWidget(self.btnUndo)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem10)
        self.btnSaveProps = QtWidgets.QPushButton(MotorPropsDialog)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnSaveProps.setIcon(icon1)
        self.btnSaveProps.setIconSize(QtCore.QSize(12, 12))
        self.btnSaveProps.setObjectName("btnSaveProps")
        self.horizontalLayout.addWidget(self.btnSaveProps)
        self.btnCancel = QtWidgets.QPushButton(MotorPropsDialog)
        self.btnCancel.setObjectName("btnCancel")
        self.horizontalLayout.addWidget(self.btnCancel)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(MotorPropsDialog)
        self.btnCancel.clicked.connect(MotorPropsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(MotorPropsDialog)
        MotorPropsDialog.setTabOrder(self.leCalibration, self.leConversion)
        MotorPropsDialog.setTabOrder(self.leConversion, self.leSlewRate)
        MotorPropsDialog.setTabOrder(self.leSlewRate, self.leAcceleration)
        MotorPropsDialog.setTabOrder(self.leAcceleration, self.leBacklash)
        MotorPropsDialog.setTabOrder(self.leBacklash, self.leLowLimit)
        MotorPropsDialog.setTabOrder(self.leLowLimit, self.leHighLimit)
        MotorPropsDialog.setTabOrder(self.leHighLimit, self.btnUndo)
        MotorPropsDialog.setTabOrder(self.btnUndo, self.btnSaveProps)
        MotorPropsDialog.setTabOrder(self.btnSaveProps, self.btnCancel)
        MotorPropsDialog.setTabOrder(self.btnCancel, self.btnCalibrate)

    def retranslateUi(self, MotorPropsDialog):
        _translate = QtCore.QCoreApplication.translate
        MotorPropsDialog.setWindowTitle(_translate("MotorPropsDialog", "Motor Properties"))
        self.label_7.setText(_translate("MotorPropsDialog", "Alias:"))
        self.lbMotorName.setText(_translate("MotorPropsDialog", "None"))
        self.label_8.setText(_translate("MotorPropsDialog", "Server: "))
        self.lbTangoServer.setText(_translate("MotorPropsDialog", "None"))
        self.leBacklash.setText(_translate("MotorPropsDialog", "0"))
        self.label_3.setText(_translate("MotorPropsDialog", "Acceleration"))
        self.label_6.setText(_translate("MotorPropsDialog", "[step/sec^2]:"))
        self.label.setText(_translate("MotorPropsDialog", "Conversion"))
        self.label_15.setText(_translate("MotorPropsDialog", "[step/unit]:"))
        self.leSlewRate.setText(_translate("MotorPropsDialog", "0"))
        self.leAcceleration.setText(_translate("MotorPropsDialog", "0"))
        self.label_2.setText(_translate("MotorPropsDialog", "Slew rate"))
        self.label_14.setText(_translate("MotorPropsDialog", "[step/sec]:"))
        self.btnCalibrate.setText(_translate("MotorPropsDialog", "Calibrate"))
        self.label_4.setText(_translate("MotorPropsDialog", "Backlash"))
        self.label_13.setText(_translate("MotorPropsDialog", "[step]:"))
        self.label_9.setText(_translate("MotorPropsDialog", "Calibration"))
        self.label_16.setText(_translate("MotorPropsDialog", "[unit]:"))
        self.leConversion.setText(_translate("MotorPropsDialog", "0.00"))
        self.leCalibration.setText(_translate("MotorPropsDialog", "0.00"))
        self.leBaseRate.setText(_translate("MotorPropsDialog", "0"))
        self.label_21.setText(_translate("MotorPropsDialog", "Base rate:"))
        self.chbUserLimits.setText(_translate("MotorPropsDialog", "user limits"))
        self.label_20.setText(_translate("MotorPropsDialog", "High limit"))
        self.label_17.setText(_translate("MotorPropsDialog", "Low limit"))
        self.label_18.setText(_translate("MotorPropsDialog", "[unit]:"))
        self.label_19.setText(_translate("MotorPropsDialog", "[unit]:"))
        self.label_11.setText(_translate("MotorPropsDialog", "High limit"))
        self.label_12.setText(_translate("MotorPropsDialog", "[unit]:"))
        self.leHighLimit.setText(_translate("MotorPropsDialog", "0.00"))
        self.label_10.setText(_translate("MotorPropsDialog", "[unit]:"))
        self.label_5.setText(_translate("MotorPropsDialog", "Low limit"))
        self.leLowLimit.setText(_translate("MotorPropsDialog", "0.00"))
        self.label_22.setText(_translate("MotorPropsDialog", "Decimals"))
        self.leDecimals.setText(_translate("MotorPropsDialog", "3"))
        self.btnAllParams.setToolTip(_translate("MotorPropsDialog", "Edit other parameters"))
        self.btnAllParams.setText(_translate("MotorPropsDialog", "All Properties..."))
        self.btnUndo.setText(_translate("MotorPropsDialog", "Undo"))
        self.btnSaveProps.setText(_translate("MotorPropsDialog", "Save"))
        self.btnCancel.setText(_translate("MotorPropsDialog", "Cancel"))

import beamline_console.gui.icons_rc



#!/usr/bin/python3
import os
import runpy

from setuptools import setup, find_packages
from setuptools.command.install import install

REQUIRES_PYTHON = '>=3.7'

REQUIRED = [
    'pyqt5', 'attrs', 'pyqtgraph >= 0.11', 'psutil', 'numpy', 'python-dateutil'
]

def abspath(*path):
    """A method to determine absolute path for a given relative path to the
    directory where this setup.py script is located"""
    setup_dir = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(setup_dir, *path)

def get_release_info():
    namespace = runpy.run_path(abspath("beamline_console/release.py"), run_name="beamline_console.release")
    return namespace["Release"]

Release = get_release_info()

class PostInstall(install):
    def run(self):
        pass

# Where the magic happens:
setup(
    name=Release.name,
    version=Release.version_long,
    description=Release.description,
    long_description=Release.long_description,
    long_description_content_type='text/markdown',
    author=Release.authors[0][0],
    author_email=Release.authors[0][1],
    download_url=Release.download_url,
    platforms=Release.platform,
    python_requires=REQUIRES_PYTHON,
    packages=find_packages(),
    package_dir={'beamline_console': 'beamline_console',},
    package_data={'beamline_console': ['LICENSE.txt',
                                      'beamline_console/beamline_console.sh',
                                      'beamline_console/beamline_console.desktop',
                                      'beamline_console/default_config/devices.xml',
                                      'beamline_console/default_config/main.cfg'
                                      ],},
    install_requires=REQUIRED,
    include_package_data=True,
    license='GPLv3',
    entry_points={'console_scripts': ['beamline_console = beamline_console:main',],},
    scripts=['beamline_console/beamline_console.sh'],
    data_files=[('share/applications', ['beamline_console/beamline_console.desktop'])],
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Development Status :: 3 - Alpha'
    ],
)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/DeviceWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DeviceWidget(object):
    def setupUi(self, DeviceWidget):
        DeviceWidget.setObjectName("DeviceWidget")
        DeviceWidget.resize(761, 374)
        self.verticalLayout = QtWidgets.QVBoxLayout(DeviceWidget)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.but_save = QtWidgets.QToolButton(DeviceWidget)
        self.but_save.setMinimumSize(QtCore.QSize(80, 80))
        self.but_save.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.but_save.setIcon(icon)
        self.but_save.setIconSize(QtCore.QSize(50, 50))
        self.but_save.setObjectName("but_save")
        self.horizontalLayout.addWidget(self.but_save)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.but_stop = QtWidgets.QPushButton(DeviceWidget)
        self.but_stop.setMinimumSize(QtCore.QSize(250, 80))
        font = QtGui.QFont()
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.but_stop.setFont(font)
        self.but_stop.setStyleSheet("color: rgb(255, 0, 0);")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/stop_red.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.but_stop.setIcon(icon1)
        self.but_stop.setIconSize(QtCore.QSize(12, 12))
        self.but_stop.setObjectName("but_stop")
        self.horizontalLayout.addWidget(self.but_stop)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.scrollArea = QtWidgets.QScrollArea(DeviceWidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 755, 280))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.wi_device_container = QtWidgets.QWidget(self.scrollAreaWidgetContents)
        self.wi_device_container.setObjectName("wi_device_container")
        self.verticalLayout_2.addWidget(self.wi_device_container)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.verticalLayout.setStretch(1, 1)

        self.retranslateUi(DeviceWidget)
        QtCore.QMetaObject.connectSlotsByName(DeviceWidget)

    def retranslateUi(self, DeviceWidget):
        _translate = QtCore.QCoreApplication.translate
        DeviceWidget.setWindowTitle(_translate("DeviceWidget", "Form"))
        self.but_save.setToolTip(_translate("DeviceWidget", "Save Beamline Snapshot"))
        self.but_stop.setToolTip(_translate("DeviceWidget", "Stop All Motors"))
        self.but_stop.setText(_translate("DeviceWidget", "STOP"))

import beamline_console.gui.icons_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/DeviceItem.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DeviceItem(object):
    def setupUi(self, DeviceItem):
        DeviceItem.setObjectName("DeviceItem")
        DeviceItem.resize(188, 146)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DeviceItem.sizePolicy().hasHeightForWidth())
        DeviceItem.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(DeviceItem)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tbStatus = QtWidgets.QToolButton(DeviceItem)
        self.tbStatus.setMaximumSize(QtCore.QSize(18, 12))
        self.tbStatus.setText("")
        self.tbStatus.setObjectName("tbStatus")
        self.horizontalLayout_2.addWidget(self.tbStatus)
        self.lbName = ClickableLabel(DeviceItem)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.lbName.setFont(font)
        self.lbName.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lbName.setObjectName("lbName")
        self.horizontalLayout_2.addWidget(self.lbName)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.lbImage = DeviceImage(DeviceItem)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbImage.sizePolicy().hasHeightForWidth())
        self.lbImage.setSizePolicy(sizePolicy)
        self.lbImage.setMinimumSize(QtCore.QSize(0, 75))
        self.lbImage.setFrameShape(QtWidgets.QFrame.Box)
        self.lbImage.setFrameShadow(QtWidgets.QFrame.Plain)
        self.lbImage.setLineWidth(1)
        self.lbImage.setText("")
        self.lbImage.setAlignment(QtCore.Qt.AlignCenter)
        self.lbImage.setObjectName("lbImage")
        self.verticalLayout.addWidget(self.lbImage)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tbLogs = QtWidgets.QToolButton(DeviceItem)
        self.tbLogs.setMinimumSize(QtCore.QSize(0, 0))
        self.tbLogs.setMaximumSize(QtCore.QSize(16777215, 18))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/page.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tbLogs.setIcon(icon)
        self.tbLogs.setIconSize(QtCore.QSize(12, 12))
        self.tbLogs.setObjectName("tbLogs")
        self.horizontalLayout.addWidget(self.tbLogs)
        self.tbCameras = QtWidgets.QToolButton(DeviceItem)
        self.tbCameras.setMaximumSize(QtCore.QSize(16777215, 18))
        self.tbCameras.setObjectName("tbCameras")
        self.horizontalLayout.addWidget(self.tbCameras)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(DeviceItem)
        QtCore.QMetaObject.connectSlotsByName(DeviceItem)

    def retranslateUi(self, DeviceItem):
        _translate = QtCore.QCoreApplication.translate
        DeviceItem.setWindowTitle(_translate("DeviceItem", "Form"))
        self.lbName.setText(_translate("DeviceItem", "[name]"))
        self.tbLogs.setToolTip(_translate("DeviceItem", "Show Logs"))
        self.tbLogs.setText(_translate("DeviceItem", "Show Logs"))
        self.tbCameras.setText(_translate("DeviceItem", "CAM"))

from experimental_control.devicecontrol.deviceimage import DeviceImage
from experimental_control.utils.clickablelabel import ClickableLabel
import beamline_console.gui.icons_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/SnapshotWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SnapshotWidget(object):
    def setupUi(self, SnapshotWidget):
        SnapshotWidget.setObjectName("SnapshotWidget")
        SnapshotWidget.resize(639, 507)
        self.verticalLayout = QtWidgets.QVBoxLayout(SnapshotWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tw_snapshots = QtWidgets.QTableWidget(SnapshotWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        font.setBold(False)
        font.setWeight(50)
        self.tw_snapshots.setFont(font)
        self.tw_snapshots.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_snapshots.setAlternatingRowColors(True)
        self.tw_snapshots.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tw_snapshots.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tw_snapshots.setObjectName("tw_snapshots")
        self.tw_snapshots.setColumnCount(0)
        self.tw_snapshots.setRowCount(0)
        self.tw_snapshots.verticalHeader().setDefaultSectionSize(100)
        self.tw_snapshots.verticalHeader().setMinimumSectionSize(100)
        self.verticalLayout.addWidget(self.tw_snapshots)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.but_apply = QtWidgets.QPushButton(SnapshotWidget)
        self.but_apply.setMinimumSize(QtCore.QSize(250, 75))
        self.but_apply.setMaximumSize(QtCore.QSize(16777215, 75))
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(False)
        font.setWeight(50)
        self.but_apply.setFont(font)
        self.but_apply.setObjectName("but_apply")
        self.horizontalLayout.addWidget(self.but_apply)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.but_delete = QtWidgets.QPushButton(SnapshotWidget)
        self.but_delete.setMinimumSize(QtCore.QSize(250, 75))
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(False)
        font.setWeight(50)
        self.but_delete.setFont(font)
        self.but_delete.setObjectName("but_delete")
        self.horizontalLayout.addWidget(self.but_delete)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout.setStretch(0, 1)

        self.retranslateUi(SnapshotWidget)
        QtCore.QMetaObject.connectSlotsByName(SnapshotWidget)

    def retranslateUi(self, SnapshotWidget):
        _translate = QtCore.QCoreApplication.translate
        SnapshotWidget.setWindowTitle(_translate("SnapshotWidget", "Form"))
        self.tw_snapshots.setSortingEnabled(True)
        self.but_apply.setText(_translate("SnapshotWidget", "Apply"))
        self.but_delete.setText(_translate("SnapshotWidget", "Delete"))

